@@ -8,6 +8,8 @@
  * does it submit to any jurisdiction.
  */
 
+#include "mars_client_config.h"
+
 #ifndef _H_MARS
 
 /* For empress <c.h>  header file */
@@ -200,7 +202,53 @@ typedef long long           long64;
 typedef unsigned long long  ulong64;
 typedef int                 boolean;
 
+#ifdef HAVE_RPC
 #include "rpcmars.h"
+#else
+typedef char *cache_t;
+typedef void *voidp_t;
+struct value {
+        /*  request part */
+        struct value *next;
+        cache_t         name;
+        /* language part */
+        struct value     *other_names;
+        struct value     *ref;
+        struct value     *expand;
+};
+typedef struct value value;
+
+struct request {
+    struct request   *next;
+    struct parameter *params;
+    cache_t         name;
+    cache_t         info;
+    cache_t         kind;
+    voidp_t          data; /* user data */
+    long             order;
+};
+typedef struct request request;
+
+struct parameter {
+    /*  request part */
+    struct parameter *next;
+    struct value     *values;
+    cache_t         name;
+    int              count;
+    struct request   *subrequest;
+    /* language part */
+    struct value     *default_values;
+    struct value     *current_values;
+    struct value     *ibm_values;
+    struct request   *interface;
+};
+typedef struct parameter parameter;
+
+struct cache_index {
+    struct request *index;
+};
+#endif
+
 /*===========================================================================*/
 /* Mars logging routines                                                     */
 /*===========================================================================*/
