@@ -3,12 +3,12 @@
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
  * In applying this licence, ECMWF does not waive the privileges and immunities
  * granted to it by virtue of its status as an intergovernmental organisation nor
  * does it submit to any jurisdiction.
  */
 
-// Baudouin Raoult - ECMWF Jan 2015
 
 #include "mir/netcdf/Calendar.h"
 
@@ -17,14 +17,10 @@ namespace mir {
 namespace netcdf {
 
 
-Calendar::Calendar() {
-}
-
-Calendar::~Calendar() = default;
-
 bool Calendar::timeAxis() const {
     return true;
 }
 
+
 }  // namespace netcdf
 }  // namespace mir
