@@ -3,15 +3,15 @@
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
  * In applying this licence, ECMWF does not waive the privileges and immunities
- * granted to it by virtue of its status as an intergovernmental organisation
- * nor does it submit to any jurisdiction.
+ * granted to it by virtue of its status as an intergovernmental organisation nor
+ * does it submit to any jurisdiction.
  */
 
 
 #include "mir/style/intgrid/Source.h"
 
-#include <iostream>
 #include <memory>
 
 #include "eckit/exception/Exceptions.h"
@@ -25,12 +25,11 @@ namespace style {
 namespace intgrid {
 
 
-static IntgridBuilder< Source > __intgrid1("source");
-static IntgridBuilder< Source > __intgrid2("SOURCE");
+static IntgridBuilder<Source> __intgrid1("source");
+static IntgridBuilder<Source> __intgrid2("SOURCE");
 
 
-Source::Source(const param::MIRParametrisation& parametrisation, long) :
-    style::Intgrid(parametrisation) {
+Source::Source(const param::MIRParametrisation& parametrisation, long) : style::Intgrid(parametrisation) {
 
     std::unique_ptr<SpectralOrder> spectralOrder(SpectralOrderFactory::build("cubic"));
     ASSERT(spectralOrder);
@@ -55,4 +54,3 @@ const std::string& Source::gridname() const {
 }  // namespace intgrid
 }  // namespace style
 }  // namespace mir
-
